'
' Copyright (c) DataEnter, Michael Kocum 1991-2006
'
' LDAPQuery.vbs queries the AD for an e-mail address
' and returns an errorlevel
'
' Usage: LDAPQuery.vbs e-mail [server|defaultNamingContext] [-uUser] [-pPassword] [-notesdomino]"
'
' Errorlevel:  0 e-mail address is valid
'              1 not used, because cscript.exe uses it
'              2 e-mail address does not exist
'              3 bad argument
'              4 error executing sql command
'
'
' History:
'
' v1.00  2004-12-28  Created
' v1.01  2004-11-28  Added optional user and password
' v1.02  2006-09-10  Added query for whole forest and support for Lotus Notes Domino
' v1.03  2006-12-02  Fix for email addresses with a single quote
' v1.04  2007-04-14  Cosmetic source changes
'
'
Option Explicit
On Error Resume Next

Const ECHO_VERBOSE = True
Const ADS_SCOPE_SUBTREE = 2

'Verify that we have an e-mail address
If WScript.Arguments.Count < 1 Then
   WScript.Echo " Usage: LDAPQuery.vbs e-mail [GCserver|defaultNamingContext] [-uUser] [-pPassword] [-notesdomino]"
   WScript.Echo "Sample: LDAPQuery.vbs postmaster@dataenter.com"
   WScript.Echo "Sample: LDAPQuery.vbs postmaster@dataenter.com exchange.dataenter.com -u MyDomain\myuser -pPassword"
   WScript.Echo "Sample: LDAPQuery.vbs postmaster@dataenter.com DC=dataenter,DC=com"
   WScript.Quit( 3 )
End If

Dim szEmail , oRootDSE , szDNC , oConnection , oCommand , szCommand , objRecordSet
Dim szUser, szPassword , szArg , iArgIdx , lNotes

' Get the e-mail
szEmail    = Replace( WScript.Arguments( 0 ) , "'" , "''" ) ' Escape single quote
szDNC      = ""
szUser     = ""    ' syntax is domain\username
szPassword = ""
lNotes     = False

' Get the optional Arguments
iArgIdx = 1
Do While iArgIdx <= Wscript.Arguments.Count - 1

   szArg = WScript.Arguments( iArgIdx )

   Select Case Left( LCase( szArg ) , 2 )
   Case "-u"
      szUser = LTrim( RTrim( Mid( szArg , 3 ) ) )
   Case "-p"
      szPassword = LTrim( RTrim( Mid( szArg , 3 ) ) )
   Case "-n"
      lNotes = True
   Case Else
      ' the DNC has no preamble
      szDNC = LTrim( RTrim( szArg ) )
   End Select

   iArgIdx = iArgIdx + 1

Loop


' No DNC, then get it from the RootDSE
If szDNC = "" Then

   If lNotes Then ' Lotus Notes Domino

      Set oRootDSE= GetObject( "LDAP://RootDSE" )
      szDNC = oRootDSE.get( "defaultNamingContext" )

   Else  ' Exchange

      ' "LDAP://" queries only the local domain,
      ' "GC://" queries the whole forest
      Set oRootDSE= GetObject( "GC://RootDSE" )
      szDNC = oRootDSE.get( "defaultNamingContext" )

   End if

End If

Set oConnection = CreateObject("ADODB.Connection")
Set oCommand    = CreateObject("ADODB.Command")
oConnection.Provider = "ADsDSOObject"
If szUser <> "" then
	oConnection.Properties("User ID") = szUser
	oConnection.Properties("Password") = szPassword
End if

oConnection.Open "Active Directory Provider"
Set oCommand.ActiveConnection = oConnection

oCommand.Properties("Page Size")     = 1000
oCommand.Properties("Timeout")       = 30
oCommand.Properties("Searchscope")   = ADS_SCOPE_SUBTREE
oCommand.Properties("Cache Results") = False

If lNotes Then ' Lotus Notes Domino

   szCommand = _
   "Select mail from 'LDAP://" & szDNC & "' " _
   & "where mail='" & szEmail & "' or uid='" & szEmail & "'"

Else ' Exchange

   ' "LDAP://" queries only the local domain,
   ' "GC://" queries the whole forest
   szCommand = _
   "Select cn,adspath,ProxyAddresses from 'GC://" & szDNC & "' " _
   & "where ProxyAddresses='SMTP:" & szEmail & "'"

End If

WScript.Echo "E-Mail: " & szEmail
WScript.Echo "   DNC: " & szDNC
If szUser <> "" then
   WScript.Echo "  User: " & szUser
   WScript.Echo "    PW: " & szPassword
End If

If ECHO_VERBOSE Then
   WScript.Echo "   SQL: " & szCommand
End If

oCommand.CommandText = szCommand
Set objRecordSet = oCommand.Execute

If Err.Number <> 0 Then
   WScript.Echo " Error: " & Err.Description
   WScript.Quit( 4 )
End If

If objRecordSet.EOF  Then

   WScript.Echo "Result: E-mail doesn't exist"
   WScript.Quit( 2 )

End If

WScript.Echo "Result: E-mail exist"

If ECHO_VERBOSE Then

   Dim aProxyAddress , szAddress

   Do Until objRecordSet.EOF

      WScript.Echo "    CN: " & objRecordSet.Fields( "cn" )
      WScript.Echo "  Path: " & objRecordSet.Fields( "adspath" )

      aProxyAddress = objRecordSet.Fields( "ProxyAddresses" ).Value

      For each szAddress in aProxyAddress
         WScript.Echo " Proxy:" & szAddress
      Next

      objRecordSet.MoveNext

   Loop

End If

WScript.Quit( 0 )


